const { spawn, spawnSync, exec } = require('child_process')
const chalk = require('chalk')

const { TEMPLATES } = require('./question')
const { isYarn, CURR_DIR } = require('./helper')
const { appConfig } = require('./config')
const { copy } = require('fs-extra')
const { updateTslintConfig } = require('./updateTslintConfig')
const { updatePackageJsonFile } = require('./updatePackageJsonFile')

module.exports.createTypescriptProject = ({ name, template, hasExistApp }) => {
  const TARGET_PATH = `${CURR_DIR}`

  if (!name) {
    name = 'abhihi-reactjs-ts-app'
  }
  name = `${TARGET_PATH}/${name}`

  if (!TEMPLATES.includes(template)) {
    template = 'typescript'
  }

  if (!hasExistApp) {
    // create react ts app
    spawn(
      isYarn ? 'yarn' : 'npx',
      [
        `${
          isYarn ? 'create react-app' : 'create-react-app'
        } ${name} --template typescript`
      ],
      { shell: true, stdio: 'inherit' }
    )
  }

  console.log(chalk.green(`🥳 Install necessary package ...`))
  spawn(
    isYarn ? 'yarn' : 'npm',
    [
      `${
        isYarn ? 'add' : 'install --save'
      } axios @reduxjs/toolkit uuid redux react-redux redux-persist react-router-dom lodash`
    ],
    { shell: true, stdio: 'inherit' }
  )
  spawn(
    isYarn ? 'yarn' : 'npm',
    [
      `${
        isYarn ? 'add --dev' : 'install --save-dev'
      } env-cmd @types/uuid @types/lodash @types/react-router-dom lint-staged node-sass prettier pretty-quick`
    ],
    { shell: true, stdio: 'inherit' }
  )

  // update tslint config
  console.log(chalk.green(`🥳 Config base URL ...`))
  updateTslintConfig()

  // Config scripts
  console.log(chalk.green(`🥳 Config scripts ...`))
  updatePackageJsonFile()

  // Run prepare
  console.log(chalk.green(`🥳 Run prepare ...`))
  spawn(isYarn ? 'yarn' : 'npm', ['run prepare'], {
    shell: true,
    stdio: 'inherit'
  })

  // copy files
  console.log(chalk.green(`🥳 Start generate file ...`))
  const packagePath = `${CURR_DIR}/node_modules/${appConfig.packageName}/source-code`
  copy(`${packagePath}/.husky`, `${TARGET_PATH}/.husky`)
  copy(`${packagePath}/.vscode`, `${TARGET_PATH}/.vscode`)
  copy(`${packagePath}/env`, `${TARGET_PATH}/public/env`)
  copy(`${packagePath}/src`, `${TARGET_PATH}/src`)

  copy(`${packagePath}/.editorconfig`, `${TARGET_PATH}/.editorconfig`)
  copy(`${packagePath}/.env.development`, `${TARGET_PATH}/.env.development`)
  copy(`${packagePath}/.env.production`, `${TARGET_PATH}/.env.production`)
  copy(`${packagePath}/.env.testing`, `${TARGET_PATH}/.env.testing`)
  copy(`${packagePath}/.env.uat`, `${TARGET_PATH}/.env.uat`)
  copy(`${packagePath}/.prettierignore`, `${TARGET_PATH}/.prettierignore`)
  copy(`${packagePath}/.prettierrc`, `${TARGET_PATH}/.prettierrc`)

  // replace src/index.tsx file
  copy(`${packagePath}/index.tsx`, `${TARGET_PATH}/index.tsx`)
  copy(`${packagePath}/App.tsx`, `${TARGET_PATH}/App.tsx`)
}
